@@ -9,35 +9,80 @@ from io import UnsupportedOperation
 
 from .structure import blob, make_blob, vercon, verlist
 from .enums import LayerFlag, BlendMode, LayerTag
+from .styles import _STYLES
 
 
 class Layer:
     def __init__(self, pxd, ID):
-        self._pxd = pxd
+        self.pxd = pxd
         self._id = ID
+        assert isinstance(ID, int)
 
         self._uuid, = pxd._db.execute(
             f"select identifier from document_layers where id = {ID};"
         ).fetchone()
 
-        self._info = dict(pxd._db.execute(
-            f"select key, value from layer_info where layer_id = {ID};"
+    @property
+    def parent(self):
+        '''The parent, which may be a Layer or a PXDFile.'''
+        # construct {child: parent} and fetch
+        children = dict(self.pxd._db.execute(
+            'select child.id, parent.id from ('
+            '  document_layers child inner join document_layers parent'
+            '  on child.parent_identifier = parent.identifier);'
         ).fetchall())
+        if self._id not in children:
+            return self.pxd
+        return self.pxd._layer(children[self._id])
+
+    @parent.setter
+    def parent(self, val):
+        if self.pxd.closed:
+            raise UnsupportedOperation('not writable')
+        if val is None or val is self.pxd:
+            ID = None
+        elif isinstance(val, Layer) and val.pxd is self.pxd:
+            ID = val._uuid
+        else:
+            raise TypeError(
+                'Parent must be own PXDFile or one of its layers.')
+
+        self.pxd._db.execute(
+            'update document_layers set parent_identifier = ?'
+            'where id = ?;',
+            (ID, self._id)
+        )
 
     def __repr__(self):
         return f'<{type(self).__name__} {repr(self.name)}>'
 
+    def _info(self, key, default=None):
+        value = self.pxd._db.execute(
+            "select value from layer_info "
+            f"where layer_id = {self._id} and key = '{key}';",
+        ).fetchone()
+        if value is None:
+            return default
+        return value[0]
+
     def _setinfo(self, key, data):
-        if self._pxd.closed:
+        if self.pxd.closed:
             raise UnsupportedOperation('not writable')
-        self._info[key] = data
-        c = self._pxd._db.cursor()
+        c = self.pxd._db.cursor()
         c.execute(
             'update layer_info set value = ?'
             'where layer_id = ? and key = ?',
             (data, self._id, key)
         )
 
+    @property
+    def mask(self):
+        for layer in self.pxd._layers(self):
+            if layer.is_mask:
+                return layer
+        else:
+            return None
+
     # Attributes
 
     @property
@@ -45,7 +90,7 @@ class Layer:
         '''
         The layer's given visible name.
         '''
-        return blob(self._info['name'])
+        return blob(self._info('name'))
 
     @name.setter
     def name(self, name: str):
@@ -62,7 +107,7 @@ class Layer:
         '''
         The layer's opacity, from 0 to 100.
         '''
-        return blob(self._info['opacity'])
+        return blob(self._info('opacity'))
 
     @opacity.setter
     def opacity(self, opacity):
@@ -76,7 +121,7 @@ class Layer:
         '''
         The position of the layer (defined as its center).
         '''
-        return tuple(blob(self._info['position']))
+        return tuple(blob(self._info('position')))
 
     @position.setter
     def position(self, pos):
@@ -88,7 +133,7 @@ class Layer:
         '''
         The position of the layer (defined as its center).
         '''
-        return tuple(blob(self._info['size']))
+        return tuple(blob(self._info('size')))
 
     @size.setter
     def size(self, size):
@@ -102,7 +147,7 @@ class Layer:
 
         A float in the range [0, 360).
         '''
-        return blob(self._info['angle']) % 360
+        return blob(self._info('angle')) % 360
 
     @angle.setter
     def angle(self, angle):
@@ -113,7 +158,7 @@ class Layer:
         '''
         The blending mode of the layer.
         '''
-        return BlendMode(blob(self._info['blendMode']))
+        return BlendMode(blob(self._info('blendMode')))
 
     @blendMode.setter
     def blendMode(self, blend):
@@ -126,7 +171,7 @@ class Layer:
         '''
         The blending mode of the layer.
         '''
-        return LayerTag(self._info['color-value'])
+        return LayerTag(self._info('color-value'))
 
     @tag.setter
     def tag(self, tag):
@@ -139,7 +184,7 @@ class Layer:
 
     @property
     def _flags(self) -> LayerFlag:
-        return LayerFlag(blob(self._info['flags']))
+        return LayerFlag(blob(self._info('flags')))
 
     @_flags.setter
     def _flags(self, val: LayerFlag):
@@ -184,18 +229,60 @@ class Layer:
     def is_mask(self, val):
         self._flag_set(LayerFlag.mask, bool(val))
 
+    @property
+    def styles(self):
+        '''
+        A list of Style objects applied to the layer, in the order applied.
+
+        Note that this must be extracted and set. For example:
+
+            layer.styles[0].opacity = 0
+
+        would not have any effect.
+        '''
+        data = self._info('styles-data')
+        if data is None:
+            return []
+        data = verlist(json.loads(data.decode()))
+        assert data['csr'] == 0
+        styles = []
+        for k in 'fsiS':
+            kind = _STYLES[k]
+            for style in data[k]:
+                style = kind._from_layer(verlist(style))
+                styles.append(style)
+        return styles
+
+    @styles.setter
+    def styles(self, val: list):
+        # attempt to extract csr, ctx
+        data = self._info('styles-data', b'[1, {}]')
+        data = verlist(json.loads(data.decode()))
+        data['csr'] = 0
+        for k in 'fsiS':
+            data[k] = []
+        for style in val:
+            k = style._tag
+            style = style._to_layer()
+            data[k].append([1, style])
+        data = json.dumps([1, data]).encode()
+        self._setinfo('styles-data', data)
+
 
 class GroupLayer(Layer):
     @property
     def children(self):
         '''
-        The children of the group layer.
+        The children of the group layer,
+        excluding any masks.
         '''
-        # TODO: avoid giving the mask, if any
-        return self._pxd._layers(self)
+        return [
+            layer for layer in self.pxd._layers(self)
+            if not layer.is_mask
+        ]
 
     def all_layers(self):
-        return self._pxd._layers(self, recurse=True)
+        return self.pxd._layers(self, recurse=True)
 
 
 class VectorLayer(Layer):
@@ -209,7 +296,7 @@ class RasterLayer(Layer):
 class TextLayer(Layer):
     def __init__(self, pxd, ID):
         Layer.__init__(self, pxd, ID)
-        data = vercon(json.loads(self._info['text-stringData']))
+        data = vercon(json.loads(self._info('text-stringData')))
         data = base64.b64decode(data['stringNSCodingData'])
         self._text = plistlib.loads(data)['$objects']
 
